// © augmentedhearing.io April 12, 2022 All Rights Reserved
//
// This is a demo example, illustrating how to call the sharpi API for voice cleanup from an Electron voice conferencing app.

// The example initializes a mediastream from the computer's default microphone and plays the denoised audio stream via the loud speaker. 
// To avoid feedback, it is recommended to use headphones or set the computer's default microphone to a virtual microphone like BlackHole, playing a noisy audio sample.

"use strict";


// 1) import methods from the sharpi API:
import {
    sharpiCleanUpVoice,
    sharpiFeatureStates
} from "./sharpiAPI/sharpi.js"; // "./sharpiDependencies/sharpi.js";


// 2) setup your daily call (for demo purpose, we just initialize a media stream here):
navigator.getUserMedia({
    audio: {
        echoCancellation: false,
        noiseSuppression: false,
        autoGainControl: false,
    }
}, function(stream) {
    76

    // 3) clean up the stream, calling the sharpi API
    let cleanStream = sharpiCleanUpVoice(stream)

    // 4) Go ahead and use the clean stream.
    // For this demo example, we play the cleaned stream via the speaker:
    let audioContext = new AudioContext()
    let cleanTrackNode = audioContext.createMediaStreamSource(cleanStream)
    cleanTrackNode.connect(audioContext.destination)


}, function(e) {
    if (e.name === "PermissionDeniedError") {
        microphoneAccessIsNotAllowed = true;
        alert("You'll need to provied access to your microphone for this web page to work.");
    }
});


// The rest is optional:
// For demo purpose, we create a UI with a standard toggle where the user can switch on and off the sharpi noise:


// /* Initialising the elements and register the event listeners. */

const noiseSuppressionOnOffButton = document.getElementById("noise_reduction_on_off")

function registerNoiseSuppressionOnOffListener(f) {
    noiseSuppressionOnOffButton.addEventListener("click", f)
}

// pass functionality to the event listeners:

// for noise suppression:
registerNoiseSuppressionOnOffListener(handleNoiseSuppressionButton)

/** sharpi noise suppression */
function handleNoiseSuppressionButton() {
    sharpiFeatureStates.suppressNoise.enable(noiseSuppressionOnOffButton.checked)

}