import {
    featureStates,
    filterStates,
    voiceTouchUpDisable,
} from './conductor.js';
import {
    denoise,
    initializeNoiseSuppressionModule
} from './denoise.js';


export var biquadNodes = {}
var audioContext
var microphone
var processingNode

var spatialNode
export var audioListener



/** Clean up the voice in the passed stream
 * @input  stream which voice needs to be cleaned
 * @return stream with a cleaned voice
 */
export function processVoiceFromStream(stream) {

    // const stream = new MediaStream([track]);

    window.AudioContext = window.AudioContext || window.webkitAudioContext;
    audioContext = new AudioContext();

    // Get access to the microphone and start pumping data through the graph.
    initializeNoiseSuppressionModule();
    processingNode = denoise(audioContext, processingNode)
    microphone = audioContext.createMediaStreamSource(stream);
    biquadNodes = createBiquads(audioContext);
    configureFilters(biquadNodes, filterStates)

    spatialNode = configureSpatialNode(audioContext)

    let destinationStream = connectAudioNodes(microphone, biquadNodes, spatialNode, processingNode, audioContext)

    return destinationStream
    // return processingNode
}



function createBiquads(audioContext) {
    var biquads = {
        fullness: audioContext.createBiquadFilter(),
        clarity: audioContext.createBiquadFilter(),
        harshness: audioContext.createBiquadFilter(),
        LFNoise: audioContext.createBiquadFilter(),
        // amplification: audioContext.createGain() }
    }
    console.log("Biquads created");

    // biquads.amplification.gain.value = 

    return biquads
}


function configureFilters(biquads, filterStates) {



    // -- Configure the Biquad fullness filter --

    biquads.fullness.type = "peaking";
    biquads.fullness.frequency.value = filterStates.centerFreq.fullness;
    biquads.fullness.gain.value = filterStates.gains.fullness;

    // -- Configure the Biquad Clarity filter --

    biquads.clarity.type = "peaking";
    biquads.clarity.frequency.value = filterStates.centerFreq.clarity;
    biquads.clarity.gain.value = filterStates.gains.clarity;

    // -- Configure the Biquad Harshness filter --

    biquads.harshness.type = "peaking";
    biquads.harshness.frequency.value = filterStates.centerFreq.harshness;
    biquads.harshness.gain.value = filterStates.gains.harshness;


    // -- Configure the Biquad Clarity filter --

    biquads.LFNoise.type = "highpass";
    biquads.LFNoise.frequency.value = 90;

    if (!featureStates.voiceTouchUp) {
        voiceTouchUpDisable()
    }


}




// -- Connect the nodes together --

function connectAudioNodes(track, biquads, spatialNode, processingNode, audioContext) {
    var destination = audioContext.createMediaStreamDestination()


    track.connect(processingNode)
    processingNode.connect(biquads.fullness)
    // track.connect(biquads.fullness)

    biquads.fullness.connect(biquads.clarity);

    biquads.clarity.connect(biquads.LFNoise);
    biquads.clarity.connect(biquads.harshness);

    // biquads.harshness.connect(destination);
    
    biquads.harshness.connect(spatialNode);
    spatialNode.connect(destination)

    console.log("audio is configured")

    console.log(audioContext.sampleRate);

    return destination.stream

}


function configureSpatialNode(audioContext) {
    audioListener = audioContext.listener

    const posX = 1;
    const posY = 0;
    const posZ = 1;

    audioListener.positionX.value = posX;
    audioListener.positionY.value = posY;
    audioListener.positionZ.value = posZ-5;


    const spatialNode = new PannerNode(audioContext, {
        panningModel: 'HRTF',
        distanceModel: 'linear',
        positionX: 0,
        positionY: 0,
        positionZ: 0,
        orientationX: 0.0,
        orientationY: 0.0,
        orientationZ: -1.0,
        refDistance: 1,
        maxDistance: 10000,
        rolloffFactor: 10,
        coneInnerAngle: 60,
        coneOuterAngle: 90,
        coneOuterGain: 0.3
    })

    return spatialNode
}

